#!/usr/bin/perl -T
use strict ;
use LWP ;
use URI::Escape ;

my ($formptr, $cgeye_ptr) = &parse_form ;
my ( $url, $host ) = &url($cgeye_ptr->{'cgeye_url'}) ;

&error('cg-eye live will not call itself')
  if ( ( $host eq $ENV{'HTTP_HOST'} ) && ( $url =~ /cg.?eye/ ) ) ;

# display HTTP request & form data
&show_request($url, $host, $formptr, $cgeye_ptr) ;
 
my $response = &submit_request($url, $host, $formptr, $cgeye_ptr) ;

&show_response($response, $url) ;

my $type = $response->content_type ;
my $body = $response->content ;

if ( $cgeye_ptr->{'cgeye_source'} ) {
  if ( $type =~ /^text/i ) {
	&show_source($body) ;
  } else {
	print "<p>Not displaying non-text response.</p>" ;
  }
}
if ( $cgeye_ptr->{'cgeye_valid'} ) {
  if ( $type =~ /^text\/html/i ) {
	&validate_html($body) ;
  } else {
	print "<p>Validation not applicable to non-HTML response.</p>" ;
  }
}
exit ;

sub show_source {
  $_ = shift ;
  print '<hr><h2>Document Contents</h2><ol>' ;
  for ( split(/[\r\n]+/,$_) ) {
    print '<li>', &escape($_) ;
  }
  print '</ol>' ;
}
sub validate_html {
  $_ = shift ;
  print "<hr><h2>HTML Validator Report</h2>" ;
  chdir "/tmp" ;
  my $tmpfile = "cgeye-$$.html" ;
  my $validator = "/usr/bin/sgmls -s -m /usr/local/lib/html-check/catalog /usr/local/lib/html-check/html.decl " ;
  umask 000 ;
  open (FILE, ">$tmpfile")
    or &error("Can't allocate temp file for validator") ;
  s/&/&amp\;/g ;		# WebTechs does this ...
  /DOCTYPE/i or do {
	print FILE '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">' ;
	print "<p><strong>Warning:</strong> No DOCTYPE declaration - assuming HTML 2.0</p>" ;
  } ;
  print FILE $_ ;
  close FILE ;
  open(VAL, "$validator $tmpfile 2>&1 && echo 'No errors!' |")
    or &error("Can't fork validator process.") ;
  print /^sgmls/ && "<p>" || '<br>', &escape($_) while ($_ = <VAL>) ;
  close VAL ;
  unlink $tmpfile ;
}
sub parse_form {
  my %FORM ;
  my %CGEYE ;

  $::PROTOCOL = $ENV{'SERVER_PROTOCOL'} || 'HTTP/1.0' ;
  $::method = $ENV{'REQUEST_METHOD'} || 'GET' ;
  my $form_data ;
  my ($name, $value) ;
  if ( $::method eq 'POST' ) {
    read(STDIN, $form_data, $ENV{'CONTENT_LENGTH'});
  } elsif ($::method eq 'GET' ) {
    $form_data = $ENV{'QUERY_STRING'} ;
  }

  my @pairs = split(/&/, $form_data);
  foreach (@pairs) {
    ($name, $value) = split('=',$_) ;

    # Un-Webify plus signs and %-encoding
    $value =~ tr/+/ /;
    $value =~ s/\%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
    $value =~ s/~!/ ~!/g;
  
    if ( $name eq "cgeye_method") {
	$::method = $value ;
    } elsif ( $name eq "cgeye_protocol") {
	$::PROTOCOL = $value ;
    } elsif ( $name =~ /^cgeye/ ) {
	$CGEYE{$name} = $value ;
    } elsif ( $value ) {
	$FORM{$name} = $value;
	$::FORMDATA .= "$name=$value" . '&' ;
    }
  }
  @pairs = split(/[\r\n]+/, $CGEYE{'cgeye_form'}) ;
  foreach(@pairs) {
    ($name, $value) = split(/=/, $_);
    $FORM{$name} = $value ;
    $::FORMDATA .= uri_escape("$name=$value") . '&' ;
  }
  chop $::FORMDATA ;
  $::headers{'Content-Length'} = length $::FORMDATA
    if $::method eq 'POST' ;

  @pairs = split(/[\r\n]+/, $CGEYE{'cgeye_headers'}) ;
  foreach(@pairs) {
    ($name, $value) = split(/:\s*/, $_);
    $name =~ s/(\w)(\w*)/\U$1\L$2\E/g ;
    $::headers{$name} = $value ;
  }
  return (\%FORM, \%CGEYE) ;
}
sub url {
  $_ = shift ;
  /^http:\/\/([\w\-\.]+)(:\d+)?(.*)$/
	or &error('Cannot accept URL ' . $_) ;
  my ($server, $port, $path) = ($1,$2,$3) ;
  my $url = "http://$server$port$path" ;
  return ($url, $server) ;
}
sub escape {
  $_ = shift ;
  s/&/&amp;/g ;
  s/</&lt;/g ;
  s/>/&gt;/g ;
  $_ ;
}
sub error {
  $_ = shift ;
  print "<h1>Error</h1>$_" ;
  exit ;
}

# ====================================================================

sub show_request {
  my ($url, $host, $formptr, $cgeye_ptr) = @_ ;

  print qq(<h2>HTTP Request</h2>
<p>cg-eye is sending the following to <strong>$url</strong>
using protocol <strong>$::PROTOCOL</strong> with method
<strong>$::method</strong>:</p>
<h3>HTTP Headers</h3><ul>
) ;
  my ($name, $value) ;
  $ENV{'HTTP_COOKIE'} and undef $ENV{'HTTP_COOKIE'} ;
  if ( $cgeye_ptr->{'cgeye_mode'} ne 'interactive' ) {
    foreach ( keys %ENV ) {
#      /^HTTP_/ or next ;
      ( $name ) =  /^HTTP_(.*)/ or next ;
      $value = &escape($ENV{$_}) ;
      $name =~ s/_/-/g ;
      $name =~ s/(\w)(\w*)/\U$1\L$2\E/g ;
      $::headers{$name} or $::headers{$name} = $value ;
    }
    foreach ( @::HEADERS ) {
      $name = $_ ;
      $value = &escape($ENV{$_}) or next ;
      $name =~ s/_/-/g ;
      $name =~ s/(\w)(\w*)/\U$1\L$2\E/g ;
      print "<li><b>$name:</b> $value" ;
      $::headers{$name} or $::headers{$name} = $value ;
    }
    $::headers{'Host'} = $host ;
    $::headers{'User-Agent'} .= " via cg-eye live" ;
  } elsif ( $::PROTOCOL ne 'HTTP/1.0' ) {
    $::headers{'Host'} = $host ;
  }

  if ( $cgeye_ptr->{'cgeye_cookie'} ) {
    $::headers{'Cookie'} = $cgeye_ptr->{'cgeye_cookie'} ;
  }
  if ( $cgeye_ptr->{'cgeye_username'} ) {
    my $auth = $cgeye_ptr->{'cgeye_username'}
	. ':' . $cgeye_ptr->{'cgeye_password'} ;
    require MIME::Base64 ;
    $::headers{'Authorization'} = "Basic " . MIME::Base64::encode_base64($auth) ;
  }

  while (( $name, $value ) = each %::headers) {
    print "<li><strong>$name</strong>: $value" ;
  }
  print "</ul><h2>Form Data</h2>This will also be sent to your server<ul>" ;
  while (( $name, $value ) = each %$formptr) {
    print qq(<li><strong>$name</strong> = $value) ;
  }
  print "</ul>" ;
}
sub submit_request {
  my ($url, $host, $formptr, $cgeye_ptr) = @_ ;
  use LWP ;
  my $agent = new LWP::UserAgent ;
  $agent->agent($::headers{'User-Agent'}) ;
  my $header = new HTTP::Headers %::headers ;
  my $req ;
  if ( $::method eq "POST" ) {
    $req = new HTTP::Request $::method, $url, $header, $::FORMDATA ;
  } elsif ( $::method eq "GET" ) {
    $url =~ /(http:\/\/.*)/ ;
    my $geturl = $1 ;
    $::FORMDATA =~ /(.*=.*)/ ;
    my $qs = $1 ;
    $geturl .= '?' . $qs if length $qs ;
    $req = new HTTP::Request $::method, $geturl, $header ;
  } else {
    &error ("cg-eye doesn't support method $::method") ;
  }
  $req->protocol($::PROTOCOL) ;

  return $cgeye_ptr->{'cgeye_follow'} eq 'Report'
	? $agent->simple_request($req)
	: $agent->request($req) ;
}
sub show_response {
  my ($response, $url) = @_ ;
  print "<hr noshade>" ;

  print "<h2>Response from $url</h2>" ,
	"<b>Status was ",$response->code," (",$response->message, ")</b>" ,
	"<tr><td colspan=2><h3>Headers returned by your server:</h3><pre>" ;
  my $head_out = $response->headers_as_string ;
  $head_out =~ s/client-date.*//mi ;
  print $head_out, "</pre>" ;
}
BEGIN {
  $ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin' ;
  $ENV{'IFS'} = '/' ;
print qq(Content-Type: text/html
Connection: close

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">
<html><head><title>cg-eye report</title></head><body>
<h1 align=center><a href="/tools/cg-eye/">cg-eye</a></h1>
) ;
@::HEADERS = qw(
	CONTENT_LENGTH
	CONTENT_TYPE
	ACCEPT
	ACCEPT_CHARSET
	ACCEPT_ENCODING
	ACCEPT_LANGUAGE
	IF_MODIFIED_SINCE
	PROXY_AUTHORIZATION
	RANGE
	REFERER
	UNLESS
	USER_AGENT
) ;
}
END {
  print qq(</body></html>) ;
}
